# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - SectionBreakout

    Break sections out of a page, making new pages for each of the sections and
    replacing them with Include macros.

    @copyright: 2011 Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL, see COPYING for details.
"""

Dependencies = ['pages']

from MoinMoin.action import ActionBase
from MoinMoin.PageEditor import PageEditor
from MoinContentSupport import ActionSupport, getHeadingDetails, escape, escattr
import re

# Action class and supporting functions.

class SectionBreakout(ActionBase, ActionSupport):

    "An action breaking sections out of pages."

    def get_form_html(self, buttons_html):
        _ = self._
        request = self.request
        page = self.page
        form = self.get_form()

        level = int(form.get("level", ["2"])[0])

        # Acquire heading details from the page.

        body = page.get_raw_body()
        heading_details = getHeadingDetails(body, level, level)

        d = {
            "buttons_html"          : buttons_html,
            "heading_level_label"   : escape(_("Heading level")),
            "found_headings_label"  : escape(_("Headings found in page")),
            "preview_label"         : escape(_("Preview")),
            "level"                 : escattr(level),
            }

        html = u'''
  <table>
    <tr>
      <td class="label">%(heading_level_label)s</td>
      <td><input type="text" name="level" value="%(level)s" size="2" /></td>
    </tr>
    <tr>
      <td class="label">%(found_headings_label)s</td>
      <td>''' % d

        for heading, level, span in heading_details:
            html += "%s<br />" % heading

        html += '''
      </td>
    </tr>
    <tr>
      <td></td>
      <td class="buttons"><input type="submit" value="%(preview_label)s" />%(buttons_html)s</td>
    </tr>
  </table>
''' % d

        return html

    def do_action(self):

        "Create the new event."

        _ = self._
        form = self.get_form()

        # A heading level must be provided.

        level = form.get("level", [None])[0]

        if not level:
            return 0, _("No heading level specified.")

        return self.break_out_headings(int(level))

    def render_success(self, msg, msgtype=None):

        """
        Render neither 'msg' nor 'msgtype' since redirection should occur
        instead.
        NOTE: msgtype is optional because MoinMoin 1.5.x does not support it.
        """

        pass

    def break_out_headings(self, level):

        """
        Break out headings at the given 'level' from the current page.
        """

        _ = self._
        request = self.request
        page = self.page
        formatter = request.formatter

        # Acquire all heading details from the page.

        page_body = page.get_raw_body()
        regions = []
        current_region_start = None

        for heading, found_level, (start, end) in getHeadingDetails(page_body):

            # Upon finding a suitable heading, begin a new region to be broken
            # out.

            if current_region_start is None and found_level >= level:
                current_region_start = heading, start

            # Upon finding a higher-level heading, end any open region.

            elif current_region_start is not None and found_level <= level:
                regions.append(current_region_start + (start,))

                # For headings at the requested level, open a new region.

                if found_level == level:
                    current_region_start = heading, start
                else:
                    current_region_start = None

        # End any open region.

        else:
            if current_region_start is not None:
                regions.append(current_region_start + (len(page_body),))

        # Make new pages for each region, rebuilding the current page body.

        retained_regions = []
        retained_region_start = 0

        for heading, start, end in regions:

            # Combine the page name and the heading to make a subpage.

            new_page_name = "%s/%s" % (page.page_name, heading)

            # Open the page for editing.

            new_page = PageEditor(request, new_page_name)

            # Save the new version of the page.

            new_page.saveText(page_body[start:end], 0)

            # Retain the preceding region for the current page.

            retained_regions.append(page_body[retained_region_start:start])

            # Insert Include macros for the broken out text.

            retained_regions.append("<<Include(%s,,editlink)>>\n" % new_page_name)

            # Start a new region to retain.

            retained_region_start = end

        # Retain any remaining text.

        else:
            retained_regions.append(page_body[retained_region_start:])

        # Edit the current page.

        edited_page = PageEditor(request, page.page_name)
        edited_page.saveText("".join(retained_regions), 0)

        # NOTE: Perhaps show a message upon failure.

        request.http_redirect(page.url(request))
        return 1, None

# Action function.

def execute(pagename, request):
    SectionBreakout(pagename, request).render()

# vim: tabstop=4 expandtab shiftwidth=4
